@@ -1,4 +1,23 @@
 
+.onLoad <- function(libname, pkgname) {
+  lookup_gh_username <<- memoize_first(lookup_gh_username)
+}
+
+is_string <- function(x) {
+  is.character(x) && length(x) == 1 && !is.na(x)
+}
+
+memoize_first <- function(fun) {
+  fun
+  cache <- list()
+  dec <- function(arg, ...) {
+    if (!is_string(arg)) return(fun(arg, ...))
+    if (is.null(cache[[arg]])) cache[[arg]] <<- fun(arg, ...)
+    cache[[arg]]
+  }
+  dec
+}
+
 gh_url <- "https://api.github.com"
 
 ok <- function(x) {
@@ -103,7 +122,20 @@ fullname <- function(fallback = NULL) {
       user <- system("git config --global user.name", intern = TRUE)
       user <- str_trim(user)
     }), silent = TRUE)
-    if (ok(user)) return(user)
+    if (ok(user)){
+      return(user)
+    } else{
+      user <- try(suppressWarnings({
+        user <- system(paste0("git config --file ",
+                              file.path(Sys.getenv("USERPROFILE"),
+                                        ".gitconfig"),
+                              " user.name"), intern = TRUE)
+        user <- str_trim(user)
+      }), silent = TRUE)
+      if(ok(user)){
+        return(user)
+      }
+    }
 
     user <- try({
       username <- username()
@@ -115,9 +147,9 @@ fullname <- function(fallback = NULL) {
       user <- sub("FullName", "", user)
       user <- str_trim(paste(user, collapse = ""))
     }, silent = TRUE)
-    
+
     if (ok(user)) return(user)
-    
+
   } else {
     user <- try({
       user <- system("getent passwd $(whoami)", intern = TRUE)
@@ -133,7 +165,20 @@ fullname <- function(fallback = NULL) {
     user <- system("git config --global user.name", intern = TRUE)
     user <- str_trim(user)
   }), silent = TRUE)
-  if (ok(user)) return(user)
+  if (ok(user)){
+    return(user)
+  } else{
+    user <- try(suppressWarnings({
+      user <- system(paste0("git config --file ",
+                            file.path(Sys.getenv("USERPROFILE"),
+                                      ".gitconfig"),
+                            " user.name"), intern = TRUE)
+      user <- str_trim(user)
+    }), silent = TRUE)
+    if(ok(user)){
+      return(user)
+    }
+  }
 
   fallback_or_stop(fallback, "Cannot determine full name")
 }
@@ -145,7 +190,7 @@ fullname <- function(fallback = NULL) {
 #' @param fallback If not \code{NULL} then this value is returned
 #'   if the email address cannot be found, instead of triggering an error.
 #' @return Email address on success. Otherwise an error is thrown.
-#' 
+#'
 #' @family user names
 #' @export
 #' @examples
@@ -158,23 +203,43 @@ email_address <- function(fallback = NULL) {
     email <- system("git config --global user.email", intern = TRUE)
     email <- str_trim(email)
   }), silent = TRUE)
-  if (ok(email)) return(email)
+  if (ok(email)){
+    return(email)
+  } else{
+    user <- try(suppressWarnings({
+      email <- system(paste0("git config --file ",
+                            file.path(Sys.getenv("USERPROFILE"),
+                                      ".gitconfig"),
+                            " user.email"), intern = TRUE)
+      email <- str_trim(email)
+    }), silent = TRUE)
+    if(ok(email)){
+      return(email)
+    }
+  }
 
   fallback_or_stop(fallback, "Cannot get email address")
 }
 
 #' Find the current user's GitHub username
 #'
-#' Searches on GitHub, for the user's email address, see
+#' Uses the \code{GITHUB_USERNAME} global variable or searches on GitHub,
+#'  for the user's email address, see
 #' \code{\link{email_address}}.
 #'
+#' This function caches the username in the current R session, and if the
+#' email address of the user is unchanged, it does not perform another
+#' GitHub query.
+#'
 #' @param token GitHub token to use. By default it uses
 #'   the \code{GITHUB_TOKEN} environment variable, if set.
+#'   If unset, uses the \code{GITHUB_PAT} environment 
+#'   variable, if set.
 #' @param fallback If not \code{NULL} then this value is returned
 #'   if the GitHub username cannot be found, instead of triggering an
 #'   error.
 #' @return GitHub username, or an error is thrown if it cannot be found.
-#' 
+#'
 #' @family user names
 #' @export
 #' @importFrom httr GET add_headers status_code content
@@ -185,8 +250,12 @@ email_address <- function(fallback = NULL) {
 #' gh_username()
 #' }
 
-gh_username <- function(token = Sys.getenv("GITHUB_TOKEN"),
+gh_username <- function(token = NULL,
                         fallback = NULL) {
+  # try reading username from global variable
+  env_gh_username <- Sys.getenv("GITHUB_USERNAME")
+  if (nzchar(env_gh_username)) return(env_gh_username)
+  
   email <- try(email_address(), silent = TRUE)
   if (ok(email)) {
     if (! grepl('@', email)) {
@@ -195,32 +264,41 @@ gh_username <- function(token = Sys.getenv("GITHUB_TOKEN"),
         "This does not seem to be an email address"
       ))
     }
-    url <- URLencode(paste0(gh_url, "/search/users?q=", email,
-                            " in:email"))
+    lookup_gh_username(email, token)
 
-    auth <- character()
-    if (token != "") auth <- c("Authorization" = paste("token", token))
+  } else {
+    fallback_or_stop(fallback, "Cannot get GitHub username")
+  }
+}
 
-    resp <- GET(
-      url,
-      add_headers("user-agent" = "https://github.com/r-lib/whoami",
-                  'accept' = 'application/vnd.github.v3+json',
-                  .headers = auth)
+lookup_gh_username <- function(email, token) {
+  url <- URLencode(paste0(gh_url, "/search/users?q=", email,
+                          " in:email"))
+
+  if(is.null(token)){
+    token <- Sys.getenv("GITHUB_TOKEN", Sys.getenv("GITHUB_PAT"))
+  }
+  
+  auth <- character()
+  if (token != "") auth <- c("Authorization" = paste("token", token))
+
+  resp <- GET(
+    url,
+    add_headers("user-agent" = "https://github.com/r-lib/whoami",
+                'accept' = 'application/vnd.github.v3+json',
+                .headers = auth)
+  )
+  if (status_code(resp) >= 300) {
+    return(fallback_or_stop(fallback, "Cannot find GitHub username"))
+  }
+  data <- fromJSON(content(resp, as = "text"), simplifyVector = FALSE)
+  if (data$total_count == 0) {
+    return(
+      fallback_or_stop(fallback, "Cannot find GitHub username for email")
     )
-    if (status_code(resp) >= 300) {
-      return(fallback_or_stop(fallback, "Cannot find GitHub username"))
-    }
-    data <- fromJSON(content(resp, as = "text"), simplifyVector = FALSE)
-    if (data$total_count == 0) {
-      return(
-        fallback_or_stop(fallback, "Cannot find GitHub username for email")
-      )
-    }
-
-    return(data$items[[1]]$login)
   }
 
-  fallback_or_stop(fallback, "Cannot get GitHub username")
+  data$items[[1]]$login
 }
 
 #' User name and full name of the current user
@@ -236,6 +314,27 @@ gh_username <- function(token = Sys.getenv("GITHUB_TOKEN"),
 #' \dontrun{
 #' whoami()
 #' }
+#' @details 
+#' For the username it tries the `LOGNAME`, `USER`, 
+#' `LNAME` and `USERNAME` environment variables first.
+#'  If these are all unset, or set to an empty string, 
+#'  then it tries running `id` on Unix-like
+#' systems and `whoami` on Windows.
+#' 
+#' For the full name of the user, it queries the system services 
+#' and also tries the user's global git configuration. 
+#' On Windows, it tries finding the global git configuration 
+#' in `Sys.getenv("USERPROFILE")` if it doesn't find it 
+#' in `Sys.getenv("HOME")` (often "Documents").
+#' 
+#' For the email address it uses the user's global git 
+#' configuration. It tries finding the global git 
+#' configuration in `Sys.getenv("USERPROFILE")` 
+#' if it doesn't find it in `Sys.getenv("HOME")`.
+#' 
+#' For the GitHub username it uses the `GITHUB_USERNAME` 
+#' environment variable then it tries searching on GitHub 
+#' for the user's email address.
 
 whoami <- function() {
   c("username" = username(),
